﻿using System;
using System.Windows.Forms;

using RipSync.RSTimeTable;

namespace RipSync.Plugins.Extensions.UST{

    public class USTReader : RipSync.Plugins.Bases.RipSyncReader{
        public USTReader()
            : base( "UST", "USTファイルを読み込む。" ) {
            Config.GetIncetance();
        }

        public override Table CreateTable() {
            return this.CreateTable( null );
        }

        public Table CreateTable( string file_name ) {
            Console.WriteLine( "USTReader+CreateTable" );
            Console.WriteLine( "    file_name=" + file_name );
            Table ret = new Table();
            if ( file_name == "" ) {
                file_name = "";
            }
            using ( USTFormSelect ufs = new USTFormSelect( file_name ) ) {
                ufs.TableName = Managers.ApplicationManager.GetInstance().TimeTableManager.GetNewName();
                if ( ufs.ShowDialog() != DialogResult.OK ) {
                    return null;
                }
                ret.Name = ufs.TableName;
                file_name = ufs.FileName;
                UstFile ust = null;
                try {
                    ust = new UstFile( file_name );
                } catch {
                    return null;
                }
                int clock = 0;
                foreach ( UstEvent ue in ust.Tracks[0].Events ) {
                    if ( ue.Lyric == "R" ) {
                        clock += (int)ue.Length;
                        continue;
                    }

                    string symbol = ue.Lyric;
                    SymbolTable.Attatch( ue.Lyric, out symbol );
                    string[] phon_symbols = symbol.Split( ' ' );
                    if ( phon_symbols.Length == 0 ) {
                        continue;
                    }

                    if ( phon_symbols.Length == 1 && phon_symbols[0] != "" ) {
                        // 単音節の場合
                        Unit add = new Unit();
                        add.Start = ust.SecFromClock( clock );
                        clock += (int)ue.Length;
                        add.End = ust.SecFromClock( clock );
                        add.Text = ue.Lyric;

                        add.Symbol = phon_symbols[0];

                        Console.WriteLine( "    u.Text,u.Symbol=" + add.Text + "," + add.Symbol );
                        if ( !ret.AddUnit( add ) ) {
                            ret.AddUnit( add, true );
                        }
                    } else if ( phon_symbols.Length > 1 && phon_symbols[0] != "" ) {
                        // 複音節の場合
                        // １音中に含まれる母音と子音の数を数えておく
                        int i, num_vowel, num_conso;
                        for ( i = 0, num_vowel = num_conso = 0; i < phon_symbols.Length; i++ ) {
                            if ( isVowel( phon_symbols[i] ) )
                                num_vowel++;
                            else
                                num_conso++;
                        }

                        double sec_clock_start = ust.SecFromClock( clock );
                        double sec_clock_end = ust.SecFromClock( clock + (int)ue.Length );

                        // 子音の発音長は、0.05秒と１音の発音長の８分の１の短い方
                        double t_conso_length, t_vowel_length;
                        t_conso_length = (sec_clock_end - sec_clock_start) / 8.0;
                        if ( t_conso_length > 0.05 ) {
                            t_conso_length = 0.05;
                        }

                        // 母音の発音長を計算する
                        double t_start = sec_clock_start;
                        t_vowel_length = (sec_clock_end - sec_clock_start - t_conso_length * (double)num_conso) / (double)num_vowel;

                        // 各音素に発音時間を振り分ける
                        for ( i = 0; i < phon_symbols.Length; i++ ) {
                            Unit add = new Unit();
                            add.Start = t_start;
                            if ( isVowel( phon_symbols[i] ) )
                                add.End = t_start + t_vowel_length;
                            else
                                add.End = t_start + t_conso_length;
                            t_start = add.End;
                            add.Text = ue.Lyric;
                            add.Symbol = phon_symbols[i];
                            Console.WriteLine( "    u.Text,u.Symbol=" + add.Text + "," + add.Symbol );
                            if ( !ret.AddUnit( add ) ) {
                                ret.AddUnit( add, true );
                            }
                        }
                        clock += (int)ue.Length;
                    }
                }
            }
            return ret;
        }

        private bool isVowel( string ss ) {
            string[] vowels = {
            /* 日本語 */
            "a", "i", "M", "e", "o",
            /* 英語 */
            "V", "{", "Q@", "@r", "@", "aI", "aU", "i:", "I", "I@", "U", "u:",
            "U@", "e", "eI", "e@", "Q", "O:", "OI", "O@", "@U"};

            int i;

            for ( i = 0; i < vowels.Length; i++ ) {
                if ( ss == vowels[i] )
                    break;
            }
            if ( i < vowels.Length )
                return true;
            else
                return false;
        }

        public override bool HasPluginConfig() {
            return true;
        }

        public override bool SavePluginConfig( string _filename ) {
            return Config.Save( _filename );
        }

        public override bool LoadPluginConfig( string _filename ) {
            return Config.Load( _filename );
        }
    }

}
